

#include <t_services.h>
#include "kernel_id.h"
#include "rx_if.h"
#include <cdefBF533.h>


#define PEEPINGTEST             // Define PEEPING test to see the top 4 of FIFO after enabling SPORT RX

#ifdef PEEPINGTEST
#define SLEEPDURATION 2000
#else
#define SLEEPDURATION 10
#endif

struct dma_descripter{
    struct dma_descripter * next_descripter;
    unsigned int * start_address;
    unsigned short config;
    unsigned short x_count;
    unsigned short x_modify;
};

static struct dma_descripter rx_dma_dsc0, rx_dma_dsc1;

unsigned int dummy_buffer0[RXIF_BUFSIZE], dummy_buffer1[RXIF_BUFSIZE];

/*
 *  RX_IF受信データの処理タスク。
 */
void rx_if_task(VP_INT exinf)
{

    vmsk_log(LOG_UPTO(LOG_INFO), LOG_UPTO(LOG_EMERG));
    syslog(LOG_NOTICE, "Sample program starts (exinf = %d).", (INT) exinf);

    syscall(serial_ctl_por(TASK_PORTID,
            (IOCTL_CRLF | IOCTL_FCSND | IOCTL_FCRCV)));

    /*
     *  タスクの起動
    act_tsk(TASK1);
    act_tsk(TASK2);
    act_tsk(TASK3);
     */

    *pSPORT0_TCR1 =
            0 << 14 |   // TCKFE,   0:sample at down edge, 1:sample at up edge
            0 << 13 |   // LATFS,   0:early frame sync, 1:late frame sync
            0 << 12 |   // LTFS,    0:Active high TFS, 1:Active low TFS
            0 << 11 |   // DITFS,   0:data dependent TFS generation, 1:data independent TFS generation
            0 << 10 |   // TFSR,    0:TFS is not required every word, 1:TFS is required every word
            0 << 9  |   // ITFS,    0:external TFS, 1:internal TFS
            0 << 4  |   // TLSBIT,  0:MSB first transmission, 1:LSB first transmission
            0 << 2  |   // TDTYPE   0:normal, 1:reserved, 2:u-law, 3:a-law
            0 << 1  |   // ITCLK,   0:external clock generation, 1:internal clock generation
            0 << 0  ;   // TSPEN    0:Tx disable, 1:Tx enable

    *pSPORT0_TCR2 =
            0 << 10 |   // TRFST,   0:left streo ch first, 1:right stereo ch first
            0 << 9  |   // TSFESE,  0:normal frame sync, 1:LR frame clock
            0 << 8  |   // TXSE,    0:secondary ch disable, 1:secondary ch enable
            0 << 0  ;   // SLEN     0-1:not allowed,2-31:Serial word length - 1


    *pSPORT0_RCR1 =
            0 << 14 |   // RCKFE,   0:sample at down edge, 1:sample at up edge
            0 << 13 |   // LARFS,   0:early frame sync, 1:late frame sync
            0 << 12 |   // LRFS,    0:Active high RFS, 1:Active low RFS
            1 << 10 |   // RFSR,    0:RFS is not required every word, 1:RFS is required every word
            0 << 9  |   // IRFS,    0:external RFS, 1:internal RFS
            0 << 4  |   // RLSBIT,  0:MSB first transmission, 1:LSB first transmission
            0 << 2  |   // RDTYPE   0:zero fill, 1:sign extend, 2:u-law, 3:a-law
            0 << 1  |   // IRCLK,   0:external clock generation, 1:internal clock generation
            0 << 0  ;   // RSPEN    0:Rx disable, 1:Rx enable

    *pSPORT0_RCR2 =
            0 << 10 |   // RRFST,   0:left streo ch first, 1:right stereo ch first
            0 << 9  |   // RSFESE,  0:normal frame sync, 1:LR frame clock
            1 << 8  |   // RXSE,    0:secondary ch disable, 1:secondary ch enable
           29 << 0  ;   // SLEN     0-1:not allowed,2-31:Serial word length - 1
    ssync();



        // 受信DMAデスクリプタを作る
    rx_dma_dsc0.next_descripter = &rx_dma_dsc1;
    rx_dma_dsc0.start_address = dummy_buffer0;
    rx_dma_dsc0.x_count = RXIF_BUFSIZE;
    rx_dma_dsc0.x_modify = sizeof(dummy_buffer0[0]);
    rx_dma_dsc0.config =
            7 <<12 |        // FLOW,    0:Stop, 1:Auto buffer, 4:Desc array, 6:Desc List small, 7:Desc, List, Large
            7 << 8 |        // NDSIZE,  the # of element of the next descripter to fetch
            0 << 7 |        // DI_EN,   0:No interrupt at the end, 1:Interrupt at the end
            1 << 6 |        // DI_SEL,  0:Interrupt at the end of outer loop, 1:Interrupt at the end of inter loop
            1 << 5 |        // RESTART, 0:Keep DMA FIFO before start, 1:Purge DMA FIFO before start
            0 << 4 |        // DMA2D,   0:Linear DMA, 1:2D DMA
            2 << 2 |        // WDSIZE,  0:8bit, 1:16bit, 2:32bit,3:reserved
            1 << 1 |        // WNR,     0:Read from memory, 1:Write to Memory
            1 << 0 ;        // DMA_EN,  0:Disable DMA, 1:Enable DMA

    rx_dma_dsc1.next_descripter = &rx_dma_dsc0;
    rx_dma_dsc1.start_address = dummy_buffer1;
    rx_dma_dsc1.x_count = RXIF_BUFSIZE;
    rx_dma_dsc1.x_modify = sizeof(dummy_buffer1[0]);
    rx_dma_dsc1.config =
            7 <<12 |        // FLOW,    0:Stop, 1:Auto buffer, 4:Desc array, 6:Desc List small, 7:Desc, List, Large
            7 << 8 |        // NDSIZE,  the # of element of the next descripter to fetch
            0 << 7 |        // DI_EN,   0:No interrupt at the end, 1:Interrupt at the end
            1 << 6 |        // DI_SEL,  0:Interrupt at the end of outer loop, 1:Interrupt at the end of inter loop
            1 << 5 |        // RESTART, 0:Keep DMA FIFO before start, 1:Purge DMA FIFO before start
            0 << 4 |        // DMA2D,   0:Linear DMA, 1:2D DMA
            2 << 2 |        // WDSIZE,  0:8bit, 1:16bit, 2:32bit,3:reserved
            1 << 1 |        // WNR,     0:Read from memory, 1:Write to Memory
            1 << 0 ;        // DMA_EN,  0:Disable DMA, 1:Enable DMA



#ifndef PEEPINGTEST
    int count = 0, event = 0;
#endif


        // RX受信を開始する
    *pDMA1_NEXT_DESC_PTR = & rx_dma_dsc0;
    *pDMA1_CONFIG = rx_dma_dsc1.config;

    *pSPORT0_RCR1 |= RSPEN;     // RX SPORT Enable
    ssync();


    /*
     *  メインループ
     */
    do {
        long int rx[4];

            // 3mS スリープ
        tslp_tsk(3);

            // FIFOにデータがたまったはずである。
            // FIFOデータを読みだして下位2bitのみ表示する
        rx[0] = dummy_buffer0[0];    // 下位2bitのみ抽出
        rx[1] = dummy_buffer0[1];    // 下位2bitのみ抽出
        rx[2] = dummy_buffer0[2];    // 下位2bitのみ抽出
        rx[3] = dummy_buffer0[3];    // 下位2bitのみ抽出

#ifndef PEEPINGTEST
        *pSPORT0_RCR1 &= ~RSPEN;     // RX SPORT disable
        if (rx[0] !=0)
        {
            event ++;
#endif
            syslog( LOG_NOTICE, "RX word : %08x,%08x,%08x,%08x", rx[0], rx[1], rx[2], rx[3] );
#ifndef PEEPINGTEST
        }
        count++;
        if (!( count % 1000 ))
            syslog( LOG_NOTICE, "trial : %d, event : %d", count, event);
#endif
            // スリープ
        tslp_tsk(SLEEPDURATION);



    } while (1);

    syslog(LOG_NOTICE, "Sample program ends.");
    kernel_exit();
}
